﻿using System.Collections.Generic;
using UnityEngine;
using System;

namespace Oxide.Plugins
{
    [Info("EasyStakes", "Noviets", "1.1.0")]
    [Description("Fill up every authorized stake with amber.")]

    public class EasyStakes : HurtworldPlugin
    {
        private new void LoadDefaultMessages()
        {
            // English
            lang.RegisterMessages(new Dictionary<string, string>
            {
                ["nopermission"] = "EasyStake: You don't have Permission to do this!",
                ["stakes"] = "EasyStakes: You're Authorized on {Count} stakes. You need {Amount} Amber to fill them all.",
                ["stakesfilled"] = "EasyStakes: Successfully filled {count} Ownership Stakes with {amount} Amber",
                ["noamber"] = "EasyStakes: You've ran out of Amber!",
                ["share"] = "EasyStakes: You've Authorized {Player} on {Amount} of your Ownership Stakes.",
                ["invalidplayer"] = "EasyStakes: Player not found.",
                ["invaliddistance"] = "EasyStakes: Distance must be a number."
            }, this);

            // Polish
            lang.RegisterMessages(new Dictionary<string, string>
            {
                ["nopermission"] = "Nie posiadasz uprawnień do tej komendy.",
                ["stakes"] = "<color=#5B5B5B>»</color> Posiadasz {Count} totemów. Aby napełnić wszystkie, potrzebujesz {Amount} bursztynu.",
                ["stakesfilled"] = "<color=#5B5B5B>»</color> Napełniono {count} totemów używając {amount} bursztynów.",
                ["noamber"] = "<color=#5B5B5B>»</color> Masz za mało bursztynu!",
                ["share"] = "<color=#5B5B5B>»</color> Posiadasz autoryzację {Player} w {Amount} totemach.",
                ["invalidplayer"] = "<color=#5B5B5B>»</color> Nie znaleziono gracza.",
                ["invaliddistance"] = "<color=#5B5B5B>»</color> Dystans musi zawierać tylko liczby."
            }, this, "pl");
        }

        private void Loaded()
        {
            permission.RegisterPermission("easystakes.use", this);
            permission.RegisterPermission("easystakes.share", this);
        }

        private PlayerSession GetSession(string name)
        {
            foreach (PlayerSession session in GameManager.Instance.GetSessions().Values)
            {
                if (session != null && session.IsLoaded)
                {
                    if (name == session.Name)
                    {
                        return session;
                    }
                }
            }
            foreach (PlayerSession session in GameManager.Instance.GetSessions().Values)
            {
                if (session != null && session.IsLoaded)
                {
                    if (session.Name.ToLower().Contains(name.ToLower()))
                    {
                        return session;
                    }
                }
            }
            return null;
        }

        private string Msg(string msg, object SteamId = null) => lang.GetMessage(msg, this, SteamId?.ToString());

        [ChatCommand("stakes")]
        private void StakeCommand(PlayerSession session, string command, string[] args)
        {
            int auth = 0;
            int AmberNeeded = 0;
            if (permission.UserHasPermission(session.SteamId.ToString(), "easystakes.use"))
            {
                if (args.Length == 0)
                {
                    foreach (OwnershipStakeServer stake in Resources.FindObjectsOfTypeAll<OwnershipStakeServer>())
                    {
                        if (stake.AuthorizedPlayers.Contains(session.Identity))
                        {
                            Inventory inv = stake.GetComponent<Inventory>() as Inventory;
                            ItemInstance slot = inv.GetSlot(0);
                            int Needs = 5 - slot.StackSize;
                            AmberNeeded += Needs;
                            auth++;
                        }
                    }
                    hurt.SendChatMessage(session, null, Msg("stakes", session.SteamId).Replace("{Count}", auth.ToString()).Replace("{Amount}", AmberNeeded.ToString()));
                }
                if (args.Length == 1)
                {
                    if (args[0].ToLower() == "fill")
                    {
                        foreach (OwnershipStakeServer stake in Resources.FindObjectsOfTypeAll<OwnershipStakeServer>())
                        {
                            if (stake.AuthorizedPlayers.Contains(session.Identity))
                            {
                                PlayerInventory pinv = session.WorldPlayerEntity.GetComponent<PlayerInventory>();
                                Inventory inv = stake.GetComponent<Inventory>() as Inventory;
                                ItemInstance slot = inv.GetSlot(0);
                                int Needs = 5 - slot.StackSize;
                                AmberNeeded += Needs;
                                auth++;
                                if ((bool)TakeAmber(pinv, Needs))
                                {
                                    slot.StackSize = 5;
                                }
                                else
                                {
                                    hurt.SendChatMessage(session, null, Msg("noamber", session.SteamId));
                                    return;
                                }
                            }
                        }
                        hurt.SendChatMessage(session, null, Msg("stakesfilled", session.SteamId).Replace("{count}", auth.ToString()).Replace("{amount}", AmberNeeded.ToString()));
                    }
                    if (args[0].ToLower() == "share")
                    {
                        if (permission.UserHasPermission(session.SteamId.ToString(), "easystakes.share"))
                        {
                            int Amount = 0;
                            PlayerSession target = GetSession(args[1]);
                            if (target != null)
                            {
                                if (args.Length == 2)
                                {
                                    foreach (OwnershipStakeServer stake in Resources.FindObjectsOfTypeAll<OwnershipStakeServer>())
                                    {
                                        if (stake.AuthorizedPlayers.Contains(session.Identity))
                                        {
                                            if (!stake.AuthorizedPlayers.Contains(target.Identity))
                                            {
                                                stake.AuthorizedPlayers.Add(target.Identity);
                                                Amount++;
                                            }
                                        }
                                    }
                                }
                                if (args.Length == 3)
                                {
                                    int Distance = 0;
                                    try { Convert.ToInt32(args[2]); } catch { hurt.SendChatMessage(session, null, Msg("invaliddistance", session.SteamId)); return; }
                                    foreach (OwnershipStakeServer stake in Resources.FindObjectsOfTypeAll<OwnershipStakeServer>())
                                    {
                                        if (Vector3.Distance(stake.gameObject.transform.position, session.WorldPlayerEntity.transform.position) <= Distance)
                                        {
                                            if (stake.AuthorizedPlayers.Contains(session.Identity))
                                            {
                                                if (!stake.AuthorizedPlayers.Contains(target.Identity))
                                                {
                                                    stake.AuthorizedPlayers.Add(target.Identity);
                                                    Amount++;
                                                }
                                            }
                                        }
                                    }
                                }
                                hurt.SendChatMessage(session, null, Msg("share", session.SteamId).Replace("{Player}", target.Name).Replace("{Amount}", Amount.ToString()));
                            }
                            else
                            {
                                hurt.SendChatMessage(session, null, Msg("invalidplayer", session.SteamId));
                            }
                        }
                        else
                        {
                            hurt.SendChatMessage(session, null, Msg("nopermission", session.SteamId));
                        }
                    }
                }
            }
            else
            {
                hurt.SendChatMessage(session, null, Msg("nopermission", session.SteamId));
            }
        }

        private object TakeAmber(PlayerInventory pinv, int amount)
        {
            var slot = pinv.Items;
            var ItemMgr = Singleton<GlobalItemManager>.Instance;
            for (var i = 0; i < pinv.Items.Length; i++)
            {
                if (slot[i] != null)
                {
                    if (slot[i].Item.ItemId == 87)
                    {
                        if (slot[i].StackSize >= amount)
                        {
                            slot[i].StackSize = slot[i].StackSize - amount;
                            pinv.Invalidate();
                            return true;
                        }
                    }
                }
            }
            return false;
        }
    }
}
